@@ -44,8 +44,9 @@ class cmd_rebase(Command):
     but the history will be different.
 
     The command takes the location of another branch on to which the branch in
-    the current working directory will be rebased. If a branch is not specified
-    then the parent branch is used, and this is usually the desired result.
+    the specified directory (by default, the current working directory) 
+    will be rebased. If a branch is not specified then the parent branch 
+    is used, and this is usually the desired result.
 
     The first step identifies the revisions that are in the current branch that
     are not in the parent branch. The current branch is then set to be at the
@@ -76,12 +77,18 @@ class cmd_rebase(Command):
         Option('pending-merges',
             help="Rebase pending merges onto local branch."),
         Option('onto', help='Different revision to replay onto.',
-            type=str)]
+            type=str),
+        Option('directory', 
+            short_name='d',
+            help="Branch to replay onto, rather than the one containing the working directory.",
+             type=str)
+        ]
 
     @display_command
     def run(self, upstream_location=None, onto=None, revision=None,
             merge_type=None, verbose=False, dry_run=False,
-            always_rebase_merges=False, pending_merges=False):
+            always_rebase_merges=False, pending_merges=False,
+            directory="."):
         from bzrlib.branch import Branch
         from bzrlib.revisionspec import RevisionSpec
         from bzrlib.workingtree import WorkingTree
@@ -97,19 +104,21 @@ class cmd_rebase(Command):
             raise BzrCommandError(
                 "--revision and --pending-merges are mutually exclusive")
 
-        wt = WorkingTree.open_containing(".")[0]
+        wt = WorkingTree.open_containing(directory)[0]
         wt.lock_write()
-        state = RebaseState1(wt)
-        if upstream_location is None:
-            if pending_merges:
-                upstream_location = "."
-            else:
-                upstream_location = wt.branch.get_parent()
-                note("Rebasing on %s" % upstream_location)
-        upstream = Branch.open_containing(upstream_location)[0]
-        upstream_repository = upstream.repository
-        upstream_revision = upstream.last_revision()
         try:
+            state = RebaseState1(wt)
+            if upstream_location is None:
+                if pending_merges:
+                    upstream_location = directory
+                else:
+                    upstream_location = wt.branch.get_parent()
+                    if upstream_location is None:
+                        raise BzrCommandError("No upstream branch specified.")
+                    note("Rebasing on %s", upstream_location)
+            upstream = Branch.open_containing(upstream_location)[0]
+            upstream_repository = upstream.repository
+            upstream_revision = upstream.last_revision()
             # Abort if there already is a plan file
             if state.has_plan():
                 raise BzrCommandError("A rebase operation was interrupted. "
@@ -141,6 +150,11 @@ class cmd_rebase(Command):
                 stop_revid = wt_parents[1]
                 assert stop_revid is not None, "stop revid invalid"
 
+            # Check for changes in the working tree.
+            if (not pending_merges and
+                wt.basis_tree().changes_from(wt).has_changed()):
+                raise UncommittedChanges(wt)
+
             # Pull required revisions
             wt.branch.repository.fetch(upstream_repository, upstream_revision)
             if onto is None:
@@ -163,7 +177,8 @@ class cmd_rebase(Command):
                 if not our_new:
                     self.outf.write("Base branch is descendant of current "
                         "branch. Pulling instead.\n")
-                    wt.pull(upstream, onto)
+                    if not dry_run:
+                        wt.pull(upstream, onto)
                     return
             # else: include extra revisions needed to make start_revid mean
             # something.
@@ -177,17 +192,12 @@ class cmd_rebase(Command):
                     not always_rebase_merges
                     )
 
-            if verbose:
+            if verbose or dry_run:
                 todo = list(rebase_todo(wt.branch.repository, replace_map))
                 note('%d revisions will be rebased:' % len(todo))
                 for revid in todo:
                     note("%s" % revid)
 
-            # Check for changes in the working tree.
-            if (not pending_merges and
-                wt.basis_tree().changes_from(wt).has_changed()):
-                raise UncommittedChanges(wt)
-
             if not dry_run:
                 # Write plan file
                 state.write_plan(replace_map)
@@ -195,7 +205,8 @@ class cmd_rebase(Command):
                 # Start executing plan
                 try:
                     rebase(wt.branch.repository, replace_map,
-                           WorkingTreeRevisionRewriter(wt, state, merge_type=merge_type))
+                        WorkingTreeRevisionRewriter(wt, state,
+                            merge_type=merge_type))
                 except ConflictsInTree:
                     raise BzrCommandError("A conflict occurred replaying a "
                         "commit. Resolve the conflict and run "
@@ -209,17 +220,22 @@ class cmd_rebase(Command):
 class cmd_rebase_abort(Command):
     """Abort an interrupted rebase."""
 
+    takes_options = [Option('directory',
+            short_name='d',
+            help="Branch to replay onto, rather than the one containing the working directory.",
+            type=str)]
+
     @display_command
-    def run(self):
+    def run(self, directory="."):
         from bzrlib.plugins.rewrite.rebase import (
             RebaseState1,
             complete_revert,
             )
         from bzrlib.workingtree import WorkingTree
-        wt = WorkingTree.open_containing('.')[0]
+        wt = WorkingTree.open_containing(directory)[0]
         wt.lock_write()
-        state = RebaseState1(wt)
         try:
+            state = RebaseState1(wt)
             # Read plan file and set last revision
             try:
                 last_rev_info = state.read_plan()[0]
@@ -233,17 +249,20 @@ class cmd_rebase_abort(Command):
 
 class cmd_rebase_continue(Command):
     """Continue an interrupted rebase after resolving conflicts."""
-    takes_options = ['merge-type']
+    takes_options = ['merge-type', Option('directory',
+            short_name='d',
+            help="Branch to replay onto, rather than the one containing the working directory.",
+            type=str)]
 
     @display_command
-    def run(self, merge_type=None):
+    def run(self, merge_type=None, directory="."):
         from bzrlib.plugins.rewrite.rebase import (
             RebaseState1,
             rebase,
             WorkingTreeRevisionRewriter,
             )
         from bzrlib.workingtree import WorkingTree
-        wt = WorkingTree.open_containing('.')[0]
+        wt = WorkingTree.open_containing(directory)[0]
         wt.lock_write()
         try:
             state = RebaseState1(wt)
@@ -281,13 +300,18 @@ class cmd_rebase_todo(Command):
 
     """
 
-    def run(self):
+    takes_options = [Option('directory',
+            short_name='d',
+            help="Branch to replay onto, rather than the one containing the working directory.",
+            type=str)]
+
+    def run(self, directory="."):
         from bzrlib.plugins.rewrite.rebase import (
             RebaseState1,
             rebase_todo,
             )
         from bzrlib.workingtree import WorkingTree
-        wt = WorkingTree.open_containing('.')[0]
+        wt = WorkingTree.open_containing(directory)[0]
         wt.lock_read()
         try:
             state = RebaseState1(wt)
@@ -309,11 +333,14 @@ class cmd_replay(Command):
 
     """
 
-    takes_options = ['revision', 'merge-type']
+    takes_options = ['revision', 'merge-type', Option('directory', 
+            short_name='d',
+            help="Branch to replay onto, rather than the one containing the working directory.",
+            type=str)]
     takes_args = ['location']
     hidden = True
 
-    def run(self, location, revision=None, merge_type=None):
+    def run(self, location, revision=None, merge_type=None, directory="."):
         from bzrlib.branch import Branch
         from bzrlib.workingtree import WorkingTree
         from bzrlib import ui
@@ -343,19 +370,21 @@ class cmd_replay(Command):
         else:
             raise BzrCommandError("--revision is mandatory")
 
-        wt = WorkingTree.open(".")
+        wt = WorkingTree.open(directory)
         wt.lock_write()
-        state = RebaseState1(wt)
-        replayer = WorkingTreeRevisionRewriter(wt, state, merge_type=merge_type)
-        pb = ui.ui_factory.nested_progress_bar()
         try:
-            for revid in todo:
-                pb.update("replaying commits", todo.index(revid), len(todo))
-                wt.branch.repository.fetch(from_branch.repository, revid)
-                newrevid = regenerate_default_revid(wt.branch.repository, revid)
-                replayer(revid, newrevid, [wt.last_revision()])
+            state = RebaseState1(wt)
+            replayer = WorkingTreeRevisionRewriter(wt, state, merge_type=merge_type)
+            pb = ui.ui_factory.nested_progress_bar()
+            try:
+                for revid in todo:
+                    pb.update("replaying commits", todo.index(revid), len(todo))
+                    wt.branch.repository.fetch(from_branch.repository, revid)
+                    newrevid = regenerate_default_revid(wt.branch.repository, revid)
+                    replayer(revid, newrevid, [wt.last_revision()])
+            finally:
+                pb.finished()
         finally:
-            pb.finished()
             wt.unlock()
 
 
@@ -391,9 +420,14 @@ class cmd_rebase_foreign(Command):
     takes_args = ['new_base?']
     takes_options = ['verbose',
         Option("idmap-file", help="Write map with old and new revision ids.",
-               type=str)]
+               type=str),
+        Option('directory', 
+            short_name='d',
+            help="Branch to replay onto, rather than the one containing the working directory.",
+            type=str)
+        ]
 
-    def run(self, new_base=None, verbose=False, idmap_file=None):
+    def run(self, new_base=None, verbose=False, idmap_file=None, directory="."):
         from bzrlib import (
             urlutils,
             )
@@ -413,11 +447,11 @@ class cmd_rebase_foreign(Command):
             )
 
         try:
-            wt_to = WorkingTree.open(".")
+            wt_to = WorkingTree.open(directory)
             branch_to = wt_to.branch
         except NoWorkingTree:
             wt_to = None
-            branch_to = Branch.open(".")
+            branch_to = Branch.open(directory)
 
         stored_loc = branch_to.get_parent()
         if new_base is None:
@@ -428,7 +462,7 @@ class cmd_rebase_foreign(Command):
                 display_url = urlutils.unescape_for_display(stored_loc,
                         self.outf.encoding)
                 self.outf.write("Using saved location: %s\n" % display_url)
-                new_bsae = Branch.open(stored_loc)
+                new_base = Branch.open(stored_loc)
         else:
             new_base = Branch.open(new_base)
 
